@@ -44,3 +44,8 @@ class DuplicateTable(Exception):
 
 class UnexpectedError(Exception):
     pass
+
+
+class DatabaseMismatch(Exception):
+    def __init__(self, dbversion, expected):
+        super().__init__(f'got version {dbversion}; expected {expected}')
