@@ -21,6 +21,24 @@ from firepit.sqlstorage import validate_name
 logger = logging.getLogger(__name__)
 
 
+CHECK_FOR_QUERIES_TABLE = (
+    "SELECT (EXISTS (SELECT *"
+    " FROM INFORMATION_SCHEMA.TABLES"
+    " WHERE TABLE_SCHEMA = %s"
+    " AND  TABLE_NAME = '__queries'))"
+)
+
+CHECK_FOR_COMMON_SCHEMA = (
+    "SELECT routines.routine_name"
+    " FROM information_schema.routines"
+    " WHERE routines.specific_schema = 'firepit_common'"
+)
+
+MATCH_FUN = '''CREATE FUNCTION firepit_common.match(pattern TEXT, value TEXT)
+RETURNS boolean AS $$
+    SELECT regexp_match(value, pattern) IS NOT NULL;
+$$ LANGUAGE SQL;'''
+
 MATCH_BIN = '''CREATE FUNCTION firepit_common.match_bin(pattern TEXT, value TEXT)
 RETURNS boolean AS $$
     SELECT regexp_match(convert_from(decode(value, 'base64'), 'UTF8'), pattern) IS NOT NULL;
@@ -31,10 +49,58 @@ RETURNS boolean AS $$
     SELECT convert_from(decode(value, 'base64'), 'UTF8') LIKE pattern;
 $$ LANGUAGE SQL;'''
 
+SUBNET_FUN = '''CREATE FUNCTION firepit_common.in_subnet(addr TEXT, net TEXT)
+RETURNS boolean AS $$
+    SELECT addr::inet <<= net::inet;
+$$ LANGUAGE SQL;'''
+
+METADATA_TABLE = ('CREATE UNLOGGED TABLE IF NOT EXISTS "__metadata" '
+                  '(name TEXT, value TEXT)')
+
+SYMTABLE = ('CREATE UNLOGGED TABLE IF NOT EXISTS "__symtable" '
+            '(name TEXT, type TEXT, appdata TEXT,'
+            ' UNIQUE(name))')
+
+QUERIES_TABLE = ('CREATE UNLOGGED TABLE IF NOT EXISTS "__queries" '
+                 '(sco_id TEXT, query_id TEXT)')
+
+CONTAINS_TABLE = ('CREATE UNLOGGED TABLE IF NOT EXISTS "__contains" '
+                  '(source_ref TEXT, target_ref TEXT, x_firepit_rank INTEGER,'
+                  ' UNIQUE(source_ref, target_ref));')
+
 COLUMNS_TABLE = ('CREATE UNLOGGED TABLE IF NOT EXISTS "__columns" '
                  '(otype TEXT, path TEXT, shortname TEXT, dtype TEXT,'
                  ' UNIQUE(otype, path));')
 
+# Bootstrap some common SDO tables
+ID_TABLE = ('CREATE UNLOGGED TABLE "identity" ('
+            ' "id" TEXT UNIQUE,'
+            ' "identity_class" TEXT,'
+            ' "name" TEXT,'
+            ' "created" TEXT,'
+            ' "modified" TEXT'
+            ')')
+
+OD_TABLE = ('CREATE UNLOGGED TABLE "observed-data" ('
+            ' "id" TEXT UNIQUE,'
+            ' "created_by_ref" TEXT,'
+            ' "created" TEXT,'
+            ' "modified" TEXT,'
+            ' "first_observed" TEXT,'
+            ' "last_observed" TEXT,'
+            ' "number_observed" BIGINT'
+            ')')
+
+INTERNAL_TABLES = [
+    METADATA_TABLE,
+    SYMTABLE,
+    QUERIES_TABLE,
+    CONTAINS_TABLE,
+    COLUMNS_TABLE,
+    ID_TABLE,
+    OD_TABLE,
+]
+
 
 def get_storage(url, session_id):
     dbname = url.path.lstrip('/')
@@ -126,6 +192,22 @@ class TuplesToTextIO:
         return result
 
 
+def _rewrite_view_def(viewname, viewdef):
+    if viewdef:
+        stmt = viewdef['definition'].rstrip(';').replace('\n', ' ')
+
+        # PostgreSQL will "expand" the original "*" to the columns
+        # that existed at that time.  We need to get the star back, to
+        # match SQLite3's behavior.
+        logger.debug('%s original:  %s', viewname, stmt)
+        stmt = _rewrite_query(stmt)
+        logger.debug('%s rewritten: %s', viewname, stmt)
+        return stmt
+
+    # Must be a table
+    return f'SELECT * FROM "{viewname}"'
+
+
 def _rewrite_query(qry):
     parts = qry.split('UNION')
     new_parts = []
@@ -195,11 +277,7 @@ class PgStorage(SqlStorage):
 
         self._execute(f'SET search_path TO "{session_id}", firepit_common;')
 
-        stmt = ("SELECT (EXISTS (SELECT *"
-                " FROM INFORMATION_SCHEMA.TABLES"
-                " WHERE TABLE_SCHEMA = %s"
-                " AND  TABLE_NAME = '__queries'))")
-        res = self._query(stmt, (session_id,)).fetchone()
+        res = self._query(CHECK_FOR_QUERIES_TABLE, (session_id,)).fetchone()
         done = list(res.values())[0] if res else False
         if not done:
             self._setup()
@@ -210,23 +288,14 @@ class PgStorage(SqlStorage):
 
     def _create_firepit_common_schema(self):
         try:
-            stmt = ("SELECT routines.routine_name"
-                    " FROM information_schema.routines"
-                    " WHERE routines.specific_schema = 'firepit_common'")
-            res = self._query(stmt).fetchall()
+            res = self._query(CHECK_FOR_COMMON_SCHEMA).fetchall()
             if not res:
                 self._execute('CREATE SCHEMA IF NOT EXISTS "firepit_common";')
                 cursor = self._execute('BEGIN;')
-                self._execute('''CREATE FUNCTION firepit_common.match(pattern TEXT, value TEXT)
-                                RETURNS boolean AS $$
-                                    SELECT regexp_match(value, pattern) IS NOT NULL;
-                            $$ LANGUAGE SQL;''', cursor=cursor)
+                self._execute(MATCH_FUN, cursor=cursor)
                 self._execute(MATCH_BIN, cursor=cursor)
                 self._execute(LIKE_BIN, cursor=cursor)
-                self._execute('''CREATE FUNCTION firepit_common.in_subnet(addr TEXT, net TEXT)
-                                RETURNS boolean AS $$
-                                    SELECT addr::inet <<= net::inet;
-                            $$ LANGUAGE SQL;''', cursor=cursor)
+                self._execute(SUBNET_FUN, cursor=cursor)
                 cursor.close()
             elif len(res) < 4:
                 # Might need to add new functions
@@ -244,22 +313,12 @@ class PgStorage(SqlStorage):
         cursor = self._execute('BEGIN;')
         try:
             # Do DB initization from base class
-            stmt = ('CREATE UNLOGGED TABLE IF NOT EXISTS "__metadata" '
-                    '(name TEXT, value TEXT);')
-            self._execute(stmt, cursor)
-            stmt = ('CREATE UNLOGGED TABLE IF NOT EXISTS "__symtable" '
-                    '(name TEXT, type TEXT, appdata TEXT,'
-                    ' UNIQUE(name));')
-            self._execute(stmt, cursor)
-            stmt = ('CREATE UNLOGGED TABLE IF NOT EXISTS "__queries" '
-                    '(sco_id TEXT, query_id TEXT);')
-            self._execute(stmt, cursor)
-            stmt = ('CREATE UNLOGGED TABLE IF NOT EXISTS "__contains" '
-                    '(source_ref TEXT, target_ref TEXT, x_firepit_rank INTEGER,'
-                    ' UNIQUE(source_ref, target_ref));')
-            self._execute(stmt, cursor)
-            self._execute(COLUMNS_TABLE, cursor)
+            for stmt in INTERNAL_TABLES:
+                self._execute(stmt, cursor)
+
+            # Record db version
             self._set_meta(cursor, 'dbversion', DB_VERSION)
+
             self.connection.commit()
             cursor.close()
         except (psycopg2.errors.DuplicateFunction, psycopg2.errors.UniqueViolation):
@@ -425,19 +484,7 @@ class PgStorage(SqlStorage):
                              " WHERE schemaname = %s"
                              " AND viewname = %s", (self.session_id, viewname))
         viewdef = cursor.fetchone()
-        if viewdef:
-            stmt = viewdef['definition'].rstrip(';').replace('\n', ' ')
-
-            # PostgreSQL will "expand" the original "*" to the columns
-            # that existed at that time.  We need to get the star back, to
-            # match SQLite3's behavior.
-            logger.debug('%s original:  %s', viewname, stmt)
-            stmt = _rewrite_query(stmt)
-            logger.debug('%s rewritten: %s', viewname, stmt)
-            return stmt
-
-        # Must be a table
-        return f'SELECT * FROM "{viewname}"'
+        return _rewrite_view_def(viewname, viewdef)
 
     def _is_sql_view(self, name, cursor=None):
         cursor = self._query("SELECT definition"
@@ -540,7 +587,11 @@ class PgStorage(SqlStorage):
         valnames = ', '.join(quoted_colnames)
 
         # Create a temp table that copies the structure of `tablename`
-        cursor.execute(f'CREATE TEMP TABLE tmp AS SELECT * FROM "{tablename}" WHERE 1=2;')
+        #cursor.execute(f'CREATE TEMP TABLE tmp AS SELECT * FROM "{tablename}" WHERE 1=2;')
+        s = ', '.join([f'"{name}" {ctype}' for name, ctype in schema.items()])
+        stmt = f'CREATE TEMP TABLE tmp ({s})'
+        logger.debug('%s', stmt)
+        cursor.execute(stmt)
 
         # Create a generator over `objs` that returns text formatted objects
         copy_stmt = f"COPY tmp({valnames}) FROM STDIN WITH DELIMITER '{SEP}'"
@@ -550,6 +601,7 @@ class PgStorage(SqlStorage):
         stmt = (f'INSERT INTO "{tablename}" ({valnames})'
                 f' SELECT {valnames} FROM tmp')
         if 'id' in colnames:
+            stmt += ' ORDER BY id'  # Avoid deadlocks
             action = 'NOTHING'
             if tablename != 'identity':
                 excluded = self._get_excluded(colnames, tablename)
